#include "Classificator.mqh"

#define __TRADESID__

class TRADESID
{
private:
  CLASSIFICATOR<ulong, ulong> OrdersID;
  CLASSIFICATOR<ulong, ulong> DealsID;

  void RefreshOrders( const bool &Refresh )
  {
    if (!Refresh || ::HistorySelect(0, INT_MAX))
    {
      const int Total = ::HistoryOrdersTotal();
      const int PrevTotalOrders = this.LastTotalOrders;

      while (this.LastTotalOrders < Total)
      {
        const ulong Ticket = ::HistoryOrderGetTicket(this.LastTotalOrders++);
        const ulong PositionID = ::HistoryOrderGetInteger(Ticket, ORDER_POSITION_ID);

        if (PositionID)
        {
          if (!this.OrdersID.Add(PositionID, Ticket))
          {
            ::Print(__FILE__ + " " + __FUNCSIG__ + ", Line =  " + (string)__LINE__ +
                    ": PositionID = " + (string)PositionID + ", Ticket = " + (string)Ticket +
                    ", HistoryOrdersTotal = " + (string)Total + ", Index(HistoryOrderGetTicket) = " + (string)(this.LastTotalOrders - 1) +
                    ", Bug = " + (string)(++this.BugsAmount));

            ::Print("PrevTotalOrders = " + (string)PrevTotalOrders);

            for (int i = ::MathMax(0, PrevTotalOrders - 1); i < Total; i++)
              ::Print("Index(HistoryOrderGetTicket) = " + (string)i + ", Ticket = " + (string)::HistoryOrderGetTicket(i));
          }

          const ulong PositionBy = ::HistoryOrderGetInteger(Ticket, ORDER_POSITION_BY_ID);

          if (PositionBy && !this.OrdersID.Add(PositionBy, Ticket))
          {
            ::Print(__FILE__ + " " + __FUNCSIG__ + ", Line =  " + (string)__LINE__ +
                    ": PositionBy = " + (string)PositionBy + ", Ticket = " + (string)Ticket +
                    ", HistoryOrdersTotal = " + (string)Total + ", Index(HistoryOrderGetTicket) = " + (string)(this.LastTotalOrders - 1) +
                    ", Bug = " + (string)(++this.BugsAmount));

            ::Print("PrevTotalOrders = " + (string)PrevTotalOrders + ", MT5build = " + (string)::TerminalInfoInteger(TERMINAL_BUILD));

            for (int i = ::MathMax(0, PrevTotalOrders - 1); i < Total; i++)
              ::Print("Index(HistoryOrderGetTicket) = " + (string)i + ", Ticket = " + (string)::HistoryOrderGetTicket(i));
          }
        }
      }
    }

    return;
  }

  void RefreshDeals( const bool &Refresh )
  {
    if (!Refresh || ::HistorySelect(0, INT_MAX))
    {
      const int Total = ::HistoryDealsTotal();
      const int PrevTotalDeals = this.LastTotalDeals;

      while (this.LastTotalDeals < Total)
      {
        const ulong Ticket = ::HistoryDealGetTicket(this.LastTotalDeals++);
        const ulong PositionID = ::HistoryDealGetInteger(Ticket, DEAL_POSITION_ID);

        if (!this.DealsID.Add(PositionID, Ticket))
        {
          ::Print(__FILE__ + " " + __FUNCSIG__ + ", Line =  " + (string)__LINE__ +
                  ": PositionID = " + (string)PositionID + ", Ticket = " + (string)Ticket +
                  ", HistoryDealsTotal = " + (string)Total + ", Index(HistoryDealGetTicket) = " + (string)(this.LastTotalDeals - 1) +
                  ", Bug = " + (string)(++this.BugsAmount));

          ::Print("PrevTotalDeals = " + (string)PrevTotalDeals);

          for (int i = ::MathMax(0, PrevTotalDeals - 1); i < Total; i++)
            ::Print("Index(HistoryDealGetTicket) = " + (string)i + ", Ticket = " + (string)::HistoryDealGetTicket(i));
        }
      }
    }

    return;
  }
protected:
  int LastTotalOrders;
  int LastTotalDeals;

  int BugsAmount;

public:
  TRADESID( void ) : LastTotalOrders(0), LastTotalDeals(0), BugsAmount(0)
  {
    VALUESID<ulong>::Reserve = 10;

    CLASSIFICATOR<ulong, ulong>::Reserve = 10000;
  }

  int GetOrdersByID( const ulong PositionID, ulong &Orders[], const bool Refresh = true )
  {
    this.RefreshOrders(Refresh);

    return(this.OrdersID.Get(PositionID, Orders));
  }

  int GetDealsByID( const ulong PositionID, ulong &Deals[], const bool Refresh = true )
  {
    this.RefreshDeals(Refresh);

    return(this.DealsID.Get(PositionID, Deals));
  }

  int GetPositionsID( ulong &PositionID[], const bool Refresh = true )
  {
    this.RefreshOrders(Refresh);

    return(this.OrdersID.GetKeys(PositionID));
  }
};